#ifndef  ELF_CONST_H_
#define  ELF_CONST_H_

#define MAJOR_NODE 0x01
#define MINOR_NODE 0x02
#define DIR_NODE   0x04

#define MAX_OPENED_FILES 3


#define UNDEFINED_NODE_ID 0xff

#define ELF_ROOT    0x08
#define ELF_FILE    0x01
#define ELF_DIR     0x04

#define NORMAL_NODE     0x00
#define RENAMED_NODE    0x01
#define OBSELETED_NODE  0x02
#define MODIFIED_NODE   0x03 
#define PHY_MAJOR_NODE  0x04

#define OBSELETE_IN_EEPROM 0x10

// #define FLAG_DIR_OFFSET 0x00

// flags of the elf inode
#define FLAG_OBSELETED 0x01
#define FLAG_DAMAGED   0x02
#define FLAG_RENAMED   0x40
#define FLAG_OPENED    0x20
#define FLAG_RELIABLE  0x10
#define FLAG_LOCKED    0x08

#define INODE_TYPE_OFFSET 0
#define INODE_VERSION_OFFSET 1
#define INODE_INODENUMBER_OFFSET 3
#define INODE_LENGTH_OFFSET 5
#define INODE_PINODE_OFFSET 9
#define INODE_FLAGS_OFFSET  11
#define INODE_NAMESIZE_OFFSET 12



#define MAX_ELF_FILE_NAME 50
#define MAX_CHILDREN_NUM 15

#define MODE_READ     0x00
#define MODE_WRITE    0x01
#define MODE_CREATE   0x02

#define DEFAULT_NAME_INDEX_PAGE   0x04

#define PAGE_CACHES       10
// is the following two values appropriate ?
#define DEFAULT_MINFREE_THRESHOLD    200
#define DEFAULT_MAXDIRTY_THRESHOLD   1000  

// elf

#define PAGE_INODE_OFFSET 0
#define PAGE_INFO_OFFSET 256
#define PAGE_CRC_OFFSET 256
#define PAGE_FLAG_OFFSET 258
#define NEXT_PAGE_OFFSET 258
#define WRITE_COUNTER_OFFSET 260

#define NODE_END_FLAG 2
#define DIRTY_FLAG 3
#define FREE_FLAG  1
#define INUSE_FLAG  0

#define WRITE_CACHE FLASH_BUFFER_2
#define READ_CACHE FLASH_BUFFER_1

#define UNREACHABLE_PAGE 2500

#define ELF_GENERAL 0x1122
#define INODE_PAGE 0xABCD 
#define INDEX_PAGE 0xCCDD


#define FS_NUM_RFDS 3


// elf_file.h
#define ELF_SUCCESS  0x00
#define ELF_ENOSPC   0x02
#define ELF_EFULL    0x03
#define ELF_WRITE_FAIL 0x01

#define FILE_DELETED  0x01

#define MAX_FILE_SIZE 500000 // 500 K is used temporarily
#define DEFAULT_INIT_FILE_SIZE 1024 // 4 pages for default allocated size
#define DATA_SIZE_PER_PAGE 256

#define ELF_FILE_END  0x01
#define ELF_FILE_CUR  0x02
#define ELF_FILE_SET  0x03

#define min(A,B) ( (A) < (B) ? (A):(B))

#define MAX_FILES 10
#define NAME_OFFSET sizeof(index_summary_t)

#define MAX_INODE_SIZE 15

// flash.h 
#define FLASH_PORT PORTD
#define FLASH_DIRE DDRD

#define FLASH_SELECT PORTA
#define FLASH_SELECT_PIN 3

#define FLASH_CLK 5
#define FLASH_OUT 2
#define FLASH_IN 3

// was flashdrive.h
#define FLASH_MAX_PAGES 2048
#define FLASH_PAGE_SIZE 264
#define FLASH_PAGE_SIZE_LOG2 8

#define FLASH_BUFFER_1 0x01
#define FLASH_BUFFER_2 0x02

#define DEFAULT_FLASH_BUFFER FLASH_BUFFER_1



// Flags 
#define SAFE_REBOOT    0x80
#define EEPROM_RESET   0x40

// summaries of the eeprom structure
#define SUMMARY_OFFSET  0x0000

// start address of the snapshot summary
#define SNAPSHOT_SUMMARY_OFFSET SUMMARY_OFFSET+sizeof(elf_eeprom_summary_t)
// start of the file system metadata
#define METADATA_OFFSET SNAPSHOT_SUMMARY_OFFSET+sizeof(elf_snapshot_summary_t)

// start of the snapshots for the files
// 0x0400 bytes currently reserved for the snapshot 
// because there are only 3 files allowed to openned for writing
// simultaneously
// shoudl come up with an util to auto resize the partition
#define SNAPSHOT_OFFSET 0x0C00

#define SNAPSHOT_INTERVAL 0x0080
#define MAX_SNAPSHOTS_PER_FILE 5

#define CURRENT_VERSION 1

// flag of the metadata summary
#define INODE_SNAPSHOT 0x01
#define NODE_SNAPSHOT  0x02
#define FLAG_METADATA_OBSELETED      0x04
#define FLAG_METADATA_RENAMED        0x08
#define FLAG_METADATA_DAMAGED        0x10

// Operation Type
#define SNAPSHOT_MODIFY 0x01  
#define SNAPSHOT_APPEND 0x02
#define SNAPSHOT_DELETE 0x03
#define SNAPSHOT_CREATE 0x04
#define SNAPSHOT_RENAME 0x05

#define DEFAULT_SNAPSHOT_NUM 40

#define EEPROM_NODE_STARTPOS_OFFSET 0x04
#define EEPROM_NODE_VERSION_OFFSET  0x08
#define EEPROM_NODE_LOCALID_OFFSET  12

// File Open Modes
#define FILE_CREATE     0x01

#define SNAPSHOT_NODE_SIZE sizeof(elf_node_t)-2

#define CACHE_SIZE  32

#define OPER_SUCCESS               0x00
#define ERROR_FILE_NAME_NOT_EXIST  0x01 
#define ERROR_FILE_OBSELETED       0x02
#define ERROR_RESOURCE_INODE_NOT_AVAILABLE 0x03
#define ERROR_RESOURCE_NODE_NOT_AVAILABLE 0x04

#endif
